@@ -1,26 +1,30 @@
-/* OPT v3.9: options parsing tool */
+/* OPT v3.19: options parsing tool */
 /*
+ * This SOFTWARE has been authored by an employee of the University of
+ * California, operator of the Los Alamos National Laboratory under
+ * Contract No. W-7405-ENG-36 with the U.S. Department of Energy. The
+ * U.S. Government has rights to use, reproduce, and distribute this
+ * SOFTWARE. Neither the Government nor the University makes any
+ * warranty, express or implied, or assumes any liability or
+ * responsibility for the use of this SOFTWARE. If SOFTWARE is modified
+ * to produce derivative works, such modified SOFTWARE should be clearly
+ * marked, so as not to confuse it with the version available from LANL.
  * 
- *     Copyright (C) 1996,1997,1998,1999 James Theiler 
- * 
- *     This program is free software; you can redistribute it and/or modify
- *     it under the terms of the GNU General Public License as published by
- *     the Free Software Foundation; either version 2 of the License, or
- *     (at your option) any later version.
- * 
- *     This program is distributed in the hope that it will be useful,
- *     but WITHOUT ANY WARRANTY; without even the implied warranty of
- *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *     GNU General Public License for more details.
- * 
- *     You should have received a copy of the GNU General Public License
- *     along with this program (it is in the file COPYING); if not, write 
- *     to the Free Software Foundation, Inc., 
- *            675 Mass Ave, Cambridge, MA 02139, USA.
+ * Additionally, this program is free software; you can distribute it
+ * and/or modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or any later version.  Accordingly, this program is
+ * distributed in the hope that it will be useful, but WITHOUT A
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details (http://www.gnu.org/copyleft/gpl.txt).
  * 
  * 
+ * The OPT package is available from
+ * http://nis-www.lanl.gov/~jt/Software
  */
-/* $Id: opt_proc.c,v 3.9 1999/11/19 17:46:57 jt Exp $ */
+
+/* $Id: opt_proc.c,v 3.26 2003/03/18 16:29:40 jt Exp $ */
 /* opt_proc.c */
 /*
  *	process options in the OPT package
@@ -30,38 +34,37 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <ctype.h>
-#ifdef convex
-#include <strings.h>
-#else
 #include <string.h>
-#endif
-#ifdef __TURBOC__
-#include <alloc.h>
-#include <stdarg.h>
-#endif
 #include "opt.h"
 #include "ag.h"
 #include "opt_p.h"
 #endif /* SINGLEFILE */
 
+#ifndef OPT_VERSION
+#define OPT_VERSION VERSION
+#endif
+
 /******************
  * Global variables 
  */
-
+OPT_PFI  opt_additional_usage_fcn=NULL;
 char	*opt_program_name = NULL;
-flag	opt_fileflag = False;
 
-char	*optfile_name = NULL;
-char    *opt_defaultfile = NULL;
-char    *opt_titlestring = NULL;
-char    *opt_usagestring = NULL;
-char	*opt_envstring=NULL;
-char	*opt_defstring=NULL;
+/* Global to this file */
 
-OPT_PFI     opt_additional_usage_fcn=NULL;
-OPT_PFI	    opt_quit_fcn=NULL;
-OPT_PFI	    opt_run_fcn=NULL;
-OPT_PFI_ARG opt_main_fcn=NULL;
+static int   opt_fileflag = OPT_FALSE;
+static int   opt_menu_enabled = OPT_TRUE;
+static char *opt_filename = NULL;
+static char *opt_defaultfile = NULL;
+static char *opt_titlestring = NULL;
+static char *opt_usagestring = NULL;
+static char *opt_envstring=NULL;
+static char *opt_defstring=NULL;
+static char *opt_pkg_version=NULL;
+
+static OPT_PFI	   opt_quit_fcn=NULL;
+static OPT_PFI	   opt_run_fcn=NULL;
+static OPT_PFI_ARG opt_main_fcn=NULL;
 
 /****************************
  * Static Function prototypes 
@@ -70,10 +73,24 @@ OPT_PFI_ARG opt_main_fcn=NULL;
 static char *append_string(char *, char *);
 static int  line2argv(int *, char ***, char *);
 static int  l2a(char *, char **);
-static flag break_word(flag, char *);
+static int  break_word(int, char *);
 static int  backup_file(char *fname);
 
+/**********************
+ * free various strings
+ */
 
+void
+opt_freestrings()
+{
+  OPT_FREE( opt_filename );
+  OPT_FREE( opt_defaultfile );
+  OPT_FREE( opt_titlestring );
+  OPT_FREE( opt_usagestring );
+  OPT_FREE( opt_envstring );
+  OPT_FREE( opt_defstring );
+  OPT_FREE( opt_pkg_version );
+}
 
 /***************************
  * set title, usage, etc.  *
@@ -81,7 +98,7 @@ static int  backup_file(char *fname);
 void
 optTitle(char *s)
 {
-    opt_titlestring = strdup(s);
+  opt_setstring(&opt_titlestring,s);
 }
 char *
 optgetTitle(void)
@@ -92,24 +109,29 @@ optgetTitle(void)
         return opt_program_name;
 }
 void
+optVersion(char *s)
+{
+  if (!ISEMPTYSTRING(s)) opt_setstring(&opt_pkg_version,s);
+}
+void
 optUsage(char *s)
 {
-    opt_usagestring = strdup(s);
+  opt_setstring(&opt_usagestring,s);
 }
 void
 optEnvVarName(char *s)
 {
-    opt_envstring = strdup(s);
+  opt_setstring(&opt_envstring,s);
 }
 void
 optDefaultString(char *s)
 {
-    opt_defstring = strdup(s);
+  opt_setstring(&opt_defstring,s);
 }
 void
 optDefaultFile(char *s)
 {
-    opt_defaultfile = strdup(s);
+  opt_setstring(&opt_defaultfile,s);
 }
 void
 optQuit(OPT_PFI fcn)
@@ -131,6 +153,31 @@ optAdditionalUsage(OPT_PFI fcn)
 {
     opt_additional_usage_fcn = fcn;
 }
+void
+optDisableMenu()
+{
+    opt_menu_enabled = OPT_FALSE;
+}
+
+
+static void
+optWriteVersion()
+{
+  char *opt_pkg;
+  opt_pkg = optgetTitle();
+  
+#ifdef VERSION
+  if (ISEMPTYSTRING(opt_pkg_version)) {
+    opt_pkg_version=VERSION;
+  }
+#endif
+
+  if (!ISEMPTYSTRING(opt_pkg_version)) {
+    opt_mess_2("%s: Version %s\n",opt_pkg,opt_pkg_version);
+  } else {
+    opt_mess_1("%s: no version information\n",opt_pkg);
+  }
+}
 
 
 
@@ -149,10 +196,11 @@ getopts(int argc, char **argv)
 	/*
 	 * Before processing, set the global variables
 	 * opt_program_name : name of routine that is running
-	 * optfile_name     : default name of options file
+	 * opt_filename     : default name of options file
 	 */
-    opt_program_name = short_progname(argv[0]);
-    optfile_name = append_string(opt_program_name,OPT_EXT);
+    if (ISEMPTYSTRING(opt_program_name))
+      opt_program_name = short_progname(argv[0]);
+    opt_filename = append_string(opt_program_name,OPT_EXT);
     opt_readline_init(opt_program_name);
                                                
 	/* Begin options processing */
@@ -191,10 +239,11 @@ opt(int *argcptr, char ***argvptr)
     /*
      * Before processing, set the global variables
      * opt_program_name : name of routine that is running
-     * optfile_name     : default name of options file
+     * opt_filename     : default name of options file
      */
-    opt_program_name = short_progname(argv[0]);
-    optfile_name = append_string(opt_program_name,OPT_EXT);
+    if (ISEMPTYSTRING(opt_program_name))
+        opt_program_name = short_progname(argv[0]);
+    opt_filename = append_string(opt_program_name,OPT_EXT);
     opt_readline_init(opt_program_name);
                                                
     /* Begin options processing */
@@ -236,8 +285,9 @@ opt(int *argcptr, char ***argvptr)
                                  */
     }
     ag_free(ag);
-    
 
+    /* Now that we are done processing, free what we no longer need */
+    OPT_FREE(opt_filename);
 }
 static char *
 append_string(char *s, char *t)
@@ -252,6 +302,15 @@ append_string(char *s, char *t)
     strcat(st,t);
     return(st);
 }
+
+/* Allow the user to explcitly set the program name */
+/* Note that this routine strips off the leading directory */
+/* so that you can safely use optProgName(argv[0]) */
+void optProgName(char* progname) {
+  opt_program_name = short_progname(progname);
+}
+
+
 char *
 short_progname(char *pname)
 {
@@ -298,7 +357,7 @@ short_progname(char *pname)
  *		system command line
  *		options file
  *		interactive command line
- *	the behavior is slightly different if the global menuflag is set
+ *	the behavior is slightly different if the global opt_menuflag is set
  */
 
 ARGVECTOR *
@@ -344,11 +403,11 @@ opt_process(int argc, char **argv)
         case ALTDELIM:
 #endif            
             if( ag_eow(ag) ) {
-                /* unadorned "-" will be treated as an undelimited option */
-                /* Try to process it as an undelimited option */
-                if( opt_undelim(ag) == 0) {
+                /* unadorned "-" will be treated as a positional option */
+                /* Try to process it as an positional option */
+	        ag_backspace(ag);
+                if( opt_parse_positional(ag) == 0) {
                     /* if not successful...then stop processing */
-                    ag_backword(ag);
                     return ag;
                 }
             } else if( ag_c(ag) == c ) {
@@ -361,60 +420,87 @@ opt_process(int argc, char **argv)
                 } else {
                     /* Special cases: --help, --optVersion, --menu, etc.. */
                     if (0==strcmp("help",ag_s(ag))) { 
-                        short_usage();
                         long_usage();
-                        if (!menuflag)
+                        if (!opt_menuflag) {
+                            ag_free(ag);
+                            optAbortRun();
+                        }
+                    } else if (0==strcmp("xml-opt-interface",ag_s(ag))) {
+                        opt_write_xml_file(stdout);
+                        if (!opt_menuflag)
                             optAbortRun();
                     } else if (0==strcmp("optVersion",ag_s(ag))) {
-#ifdef VERSION                        
-                        opt_mess_1("OPT Version %s\n",VERSION);
+#ifdef OPT_VERSION                        
+                        opt_mess_1("OPT Version %s\n",OPT_VERSION);
 #endif
-                        if (!menuflag)
+                        if (!opt_menuflag) {
+                            ag_free(ag);
                             optAbortRun();
+                        }
                     } else if (0==strcmp("menu",ag_s(ag))) { /* --menu */
-                        if ( !menuflag )      /* If not called already */
-                            ag_w_advance(ag); /* Swallow the '--menu' */
-                            opt_menu();       /* and call the menu */	
+			if (!opt_menu_enabled) {
+			    opt_warning("menu not available for this application");
+			    ag_w_advance(ag);
+			}
+		        else {
+			    if ( !opt_menuflag )      /* If not called already */
+				ag_w_advance(ag); /* Swallow the '--menu' */
+			    opt_menu();       /* and call the menu */	
+		        }
+                    } else if (0==strcmp("version",ag_s(ag)) &&
+                               !ISEMPTYSTRING(opt_pkg_version)) {
+                      optWriteVersion();
+                      if (!opt_menuflag) {
+                          ag_free(ag);
+                          optAbortRun();
+                      }
                     } else {
                         /* if a generic "--longoption" */
-                        opt_longdelim(ag);
+                        opt_parse_longdelim(ag);
                     }
                 }
                 /* if "-something" */
             } else {
                 iword = ag_w_number(ag);
                 while( iword==ag_w_number(ag) && !ag_eow(ag) )
-                    opt_delim(ag);
+                    opt_parse_delim(ag);
             }
             break;
 
         case OPTFROMFILE:
             if (! opt_fromfname(argnext(ag))) {
-                opt_warn_1("Cannot open options file [%s]\n",optfile_name);
+                opt_warn_1("Cannot open options file [%s]\n",opt_filename);
             }
             break;
             
         case OPTTOFILE:
             opt_tofname(argnext(ag));
+            if ( !opt_menuflag )
+              opt_quit();
             break;
 
         case HELPCH:
-            if( !opt_fileflag)
-                opt_help(argnext(ag));
+	    if( !opt_fileflag)
+		opt_help(argnext(ag));
             break;
 
         case INTERACT:
-            if ( !menuflag )		/* If not called already */
+            if ( !opt_menuflag )		/* If not called already */
                 opt_menu();				/* Call the menu */	
             else {						/* otherwise */ 
-                menuflag=False;			/* turn off menu */
+                opt_menuflag=OPT_FALSE;			/* turn off menu */
                 ag_clear(ag);
             }
             break;
         case IGNOREEOL:
             ag_clear(ag);
             break;
+
         case RUN:
+	    /* If we're on the command line then treat this as a positional option */
+	    if (!(opt_menuflag || opt_fileflag))
+		goto POSITIONAL;    /* I used goto! Hey, that felt ... good */
+
             /* Give precedence to optMain, then optRun */
             if ( opt_main_fcn != NULL ) {
                 int retval;
@@ -446,8 +532,8 @@ opt_process(int argc, char **argv)
                  * from the menu, then exit the menu, so that
                  * the routine is run, and then program will exit
                  */
-                if ( menuflag ) {
-                    menuflag=False;			/* turn off menu */
+                if ( opt_menuflag ) {
+                    opt_menuflag=OPT_FALSE;			/* turn off menu */
                     ag_clear(ag);
                 } else {
                     /* RUN called from a file or the command line:
@@ -457,7 +543,11 @@ opt_process(int argc, char **argv)
                 }
                 break;
 				
+	    /* Note that the QUITCH '.' is quite a common positional option! */
+            /* So we only treat it as a quit signal if using the menu */
             case QUITCH:
+		if (!opt_menuflag)
+		    goto POSITIONAL;   /* There I go again... */
                 opt_quit();
                 break;
 				
@@ -465,8 +555,8 @@ opt_process(int argc, char **argv)
                 /* If there is a BANG as the first character of a
                  * menuline, then we shouldn't even get here.
                  */
-                if (!menuflag)
-                    opt_warning("Shell can only be invoked from the menu");
+                if (!opt_menuflag)
+		    goto POSITIONAL;     /* Can't ... stop ... */
                 else {
                     opt_warn_1("Shell to system invalid unless \'%c\' is the first character in the line\n",BANG);
                 }
@@ -480,9 +570,10 @@ opt_process(int argc, char **argv)
             case '\t':
             case '\0':
                 break;
-				
+		
+	    POSITIONAL:
             default:
-                /* in the default case, option may be undelimited */
+                /* in the default case, option may be positional */
                 /* ---------------------------------------------- */
                 if(DEBUG) ag_fprint(stderr,ag);
                 ag_backspace(ag);
@@ -490,10 +581,9 @@ opt_process(int argc, char **argv)
                     opt_mess_1("opt_proc: in default section [%s]\n",ag_s(ag));
                     ag_fprint(stderr,ag);
                 }
-                /* Try to process it as an undelimited option */
-                if( opt_undelim(ag) == 0) {
+                /* Try to process it as an positional option */
+                if( opt_parse_positional(ag) == 0) {
                     /* if not successful...then stop processing */
-                    ag_backword(ag);
                     if(DEBUG) ag_fprint(stderr,ag);
                     return ag;
                 }
@@ -515,15 +605,18 @@ opt_process(int argc, char **argv)
 int
 opt_lineprocess(char *line)
 {
-    int nargc;
+    int i,nargc;
     char **nargv;
 
     if( line==NULL || *line=='\0') return(0);
  
     line2argv(&nargc,&nargv,line);
     opt_process(nargc,nargv);
-    /* at this point, we leak the copy of doubly-terminated
-     * line we made in line2argv */
+    /* Now free nargv */
+    for (i=0; i<nargc; ++i) {
+      free(nargv[i]);
+    }
+    free(nargv);    
     return(nargc);
 }
 /*	line2argv(): converts a one-line string into an argument vector */
@@ -543,7 +636,7 @@ line2argv(int *pargc, char ***pargv, char *line)
  	 * not be corrupted -- also so that input line
  	 * doesn't have to be double null terminated
  	 */
-    lincp = (char *)calloc(strlen(line)+2,sizeof(char));
+    lincp = (char *)malloc((strlen(line)+2)*sizeof(char));
     strcpy(lincp,line);
 	/*
 	 * Next step is to double null terminate 
@@ -556,10 +649,9 @@ line2argv(int *pargc, char ***pargv, char *line)
     *pargv = (char **)malloc(((*pargc)+1)*sizeof(char *));
     	/* fill arg vector with words in line */
     l2a(lincp,*pargv);
-	/*
-	 * Note that lincp cannot be freed since
-	 * that space is used by pargv
-	 */
+        /* we are done with lincp, free it... */
+    free(lincp);
+
     return(*pargc);
 }
 
@@ -587,15 +679,15 @@ line2argv(int *pargc, char ***pargv, char *line)
 static int
 l2a(char *line, char **argv)
 {
-    flag inquote=False;	/* flag: inside quotation */
-    int	 iarg;
+    int inquote=OPT_FALSE;	/* flag: inside quotation */
+    int iarg;
     
     for(iarg=0; *line != '\0'; ++iarg) {
         if(!inquote) {
             while( isspace(*line) )
                 ++line;	/* skip leading blank spaces */
             if( *line == QUOTE ) {
-                inquote=True;
+                inquote=OPT_TRUE;
                 ++line;	/* skip past leading quote */
             }
         }
@@ -606,7 +698,7 @@ l2a(char *line, char **argv)
             ++line;
 		
         if( *line==QUOTE ) {		/* toggle quote */ 
-            inquote = (inquote==True ? False : True );
+            inquote = (inquote==OPT_TRUE ? OPT_FALSE : OPT_TRUE );
             if(argv==NULL)	++line;	/* skip past quote */
         }
 		
@@ -619,17 +711,26 @@ l2a(char *line, char **argv)
                 ++line;		/* skip trailing blanks */
 		
     }
+    /* We do the following to make the line free-able */
+    if (argv != NULL) {
+      int i;
+      for (i=0; i<iarg; ++i) {
+        argv[i] = strdup(argv[i]);
+      }
+    }
+    /* Of course we will still need to free the argv's... */
+    
     return(iarg);
 }
-static flag
-break_word(flag inquote, char *line)
+static int
+break_word(int inquote, char *line)
 {
     if( *line == '\0' || *line==QUOTE )
-        return(True);
+        return(OPT_TRUE);
     if( !inquote && isspace(*line) )
-        return(True);
+        return(OPT_TRUE);
     
-    return(False);
+    return(OPT_FALSE);
 }
 
 
@@ -637,23 +738,26 @@ break_word(flag inquote, char *line)
 	/* write out a usage statement */
 	/* --------------------------- */
 
+/* A global function which can be used to programatically */
+/* print the usage. */
+void optPrintUsage() {
+  long_usage();
+}
+
+
 void
 short_usage(void)
 {
-    if(opt_titlestring != NULL)
-        opt_message(opt_titlestring);
-    if(opt_usagestring != NULL)
-        opt_mess_2("Usage: %s %s\n",opt_program_name,opt_usagestring);
-    else
-        opt_mess_1("Usage: %s [options]\n",opt_program_name);
-    
-    opt_mess_1("To invoke the menu, type:\n\t%s --menu\n",
-               opt_program_name);
+  opt_message(opt_titlestring);
+  opt_mess_1("Usage: %s ",opt_program_name);
+  opt_mess_1("%s\n", opt_usagestring ? opt_usagestring : "[options] ");
+  if (opt_menu_enabled)
+      opt_mess_1("To invoke the menu, type:\n\t%s --menu\n", opt_program_name);
 }	
 void
 long_usage(void)
 {
-    opt_message("The options are:\n");
+    short_usage();
     opt_usage();
 }
 
@@ -667,11 +771,11 @@ opt_tofname(char *fname)
     FILE	*fp;
     
     if( *fname != OPTTOFILE ) {
-	free((char *)optfile_name);
-	optfile_name = strdup(fname);
+	free((char *)opt_filename);
+	opt_filename = strdup(fname);
     }
-    backup_file(optfile_name);
-    fp = fopen(optfile_name,"w");
+    backup_file(opt_filename);
+    fp = fopen(opt_filename,"w");
 
     if(fp==NULL) {
 	opt_mess_1("cant open opt file [%s] for writing\n",fname);
@@ -712,20 +816,20 @@ backup_file(char *fname)
 	/* get and process options from a named file */
 	/* ----------------------------------------- */
 
-#define	MAXOPLINE	256
+#define	MAXOPLINE	2560
 int
 opt_fromfname(char *fname)
 {
     FILE	*fp;
     char	line[MAXOPLINE+2];	/* leave room for termination */
-    flag        tmp_fileflag;
+    int         tmp_fileflag;
     
     if (ISEMPTYSTRING(fname)) {
         opt_warn_1("%cfile.opt fails to specify file.opt\n",OPTFROMFILE);
         return 0;
     }
     if (*fname == OPTFROMFILE) {      /* if '@@' */
-        fp = fopen(optfile_name,"r");
+        fp = fopen(opt_filename,"r");
     } else {                          /* if '@file.opt' */
         fp = fopen(fname,"r");
     }
@@ -734,7 +838,7 @@ opt_fromfname(char *fname)
     }
 
     tmp_fileflag = opt_fileflag;
-    opt_fileflag = True;
+    opt_fileflag = OPT_TRUE;
     while( opt_getline(line,fp) > 0 )
     	opt_lineprocess(line);
     opt_fileflag = tmp_fileflag;
@@ -774,19 +878,19 @@ opt_help(char *s)
         opt_mess_2("\t%c %-20s\n",HELPCH,"Help");
         opt_mess_2("\t%c %-20s\n",RUN,"Run program and return to menu");
         opt_mess_2("\t%c %-20s\n",BANG,"Shell to Operating System"); 
-        if (menuflag)
+        if (opt_menuflag)
             opt_mess_2("\t%c %-20s\n",INTERACT,"Exit menu"); 
         else
             opt_mess_2("\t%c %-20s\n",INTERACT,"Invoke menu"); 
         opt_mess_2("\t%c %-20s\n",ADDITIONAL_OPTS,"Additional options");
         opt_mess_3("\t%c<filename> %-20s [%s]\n",OPTFROMFILE,
-                   "Get options from file",optfile_name);
+                   "Get options from file",opt_filename);
         opt_mess_4("\t%c%c %-2s [%s]\n",OPTFROMFILE,OPTFROMFILE,
-                   "Get options from file",optfile_name);
+                   "Get options from file",opt_filename);
         opt_mess_2("\t%c<filename> %-20s\n",OPTTOFILE,
                    "Put options in file");
         opt_mess_4("\t%c%c %-2s [%s]\n",OPTTOFILE,OPTTOFILE,
-                   "Put options in file",optfile_name);
+                   "Put options in file",opt_filename);
         opt_mess_2("\t%c %-20s\n",QUITCH,"Quit");
     } else {
         if( s[1]=='\0')
@@ -806,6 +910,9 @@ opt_quit(void)
 
     if( opt_quit_fcn != NULL )
         retval = (*opt_quit_fcn)();
+
+    /* Let's free up memory */
+    opt_free();
     /* Do we want to do anything with the retval? */
     /* eg, exit vs optAbortRun?? */
     exit(opt_exit_number);
