@@ -1,24 +1,27 @@
-/* OPT v3.9: options parsing tool */
+/* OPT v3.19: options parsing tool */
 /*
+ * This SOFTWARE has been authored by an employee of the University of
+ * California, operator of the Los Alamos National Laboratory under
+ * Contract No. W-7405-ENG-36 with the U.S. Department of Energy. The
+ * U.S. Government has rights to use, reproduce, and distribute this
+ * SOFTWARE. Neither the Government nor the University makes any
+ * warranty, express or implied, or assumes any liability or
+ * responsibility for the use of this SOFTWARE. If SOFTWARE is modified
+ * to produce derivative works, such modified SOFTWARE should be clearly
+ * marked, so as not to confuse it with the version available from LANL.
  * 
- *     Copyright (C) 1996,1997,1998,1999 James Theiler 
- * 
- *     This program is free software; you can redistribute it and/or modify
- *     it under the terms of the GNU General Public License as published by
- *     the Free Software Foundation; either version 2 of the License, or
- *     (at your option) any later version.
- * 
- *     This program is distributed in the hope that it will be useful,
- *     but WITHOUT ANY WARRANTY; without even the implied warranty of
- *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *     GNU General Public License for more details.
- * 
- *     You should have received a copy of the GNU General Public License
- *     along with this program (it is in the file COPYING); if not, write 
- *     to the Free Software Foundation, Inc., 
- *            675 Mass Ave, Cambridge, MA 02139, USA.
+ * Additionally, this program is free software; you can distribute it
+ * and/or modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or any later version.  Accordingly, this program is
+ * distributed in the hope that it will be useful, but WITHOUT A
+ * WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ * for more details (http://www.gnu.org/copyleft/gpl.txt).
  * 
  * 
+ * The OPT package is available from
+ * http://nis-www.lanl.gov/~jt/Software
  */
 /****************************************
 *	Show up a weirdness in opt hooks.
@@ -54,6 +57,8 @@ main( int argc, char *argv[] )
 
 	opt( &argc, &argv ) ;
 
+        opt_free();
+
         return 0;
 }
 /* ____________________________________ */
